/************************************************************************//**
*   @file:    AD855X.c  
*   @brief:
*   
*****************************************************************************
*   Copyright (c) 2011 Analog Devices Inc.
*   License Agreement (if necessary)
*   Legal Disclaimer (if necessary)
*
****************************************************************************/


/***************************************************************************/
/***************************** Include Files *******************************/
/***************************************************************************/
#include "ADuC841.h"
#include "ADuC841Driver.h"
#include "AD855X.h"
#include "MCP23017.h"
#include "ControlChannel.h"
#include "Protocol.h"



/***************************************************************************/
/************************ Functions Declarations ***************************/
/***************************************************************************/


void AD855XDataPacket(AD855XDataStructure* AD855XData, unsigned char function, unsigned char parameter, unsigned char value )
{
	unsigned char i;

	

	AD855XData->start = START;
	AD855XData->function = function;
	AD855XData->parameter = parameter;
	AD855XData->dummy = DUMMY;
	AD855XData->value = value;
	AD855XData->end = STOP;

	for(i=0; i<12; i++)
	{
		if((AD855XData->start & 0x800) == 0x800)
			AD855XData->SerialBit[i] = 1;
		else
			AD855XData->SerialBit[i] = 0;
		AD855XData->start <<=1;
	}
	for(i=0; i<2; i++)
	{
		if((AD855XData->function & 0x02) == 0x02)
			AD855XData->SerialBit[i+12] = 1;
		else
			AD855XData->SerialBit[i+12] = 0;
		AD855XData->function <<=1;
	}
	for(i=0; i<2; i++)
	{
		if((AD855XData->parameter & 0x02) == 0x02)
			AD855XData->SerialBit[i+14] = 1;
		else
			AD855XData->SerialBit[i+14] = 0;
		AD855XData->parameter <<=1;
	}
	for(i=0; i<2; i++)
	{
		if((AD855XData->dummy & 0x02) == 0x02)
			AD855XData->SerialBit[i+16] = 1;
		else
			AD855XData->SerialBit[i+16] = 0;
		AD855XData->dummy <<=1;
	}
	for(i=0; i<8; i++)
	{
		if((AD855XData->value & 0x80) == 0x80)
			AD855XData->SerialBit[i+18] = 1;
		else
			AD855XData->SerialBit[i+18] = 0;
		AD855XData->value <<=1;
	}
	for(i=0; i<12; i++)
	{
		if((AD855XData->end & 0x800) == 0x800)
			AD855XData->SerialBit[i+26] = 1;
		else
			AD855XData->SerialBit[i+26] = 0;
		AD855XData->end <<= 1;
	}
}


void AD855XChangeSenseCurrent(unsigned char value)
{
	xdata AD855XDataStructure AD855XDataTemp;

	unsigned char i;

	EA = 0;	 /* clear interrupts */

	AD855XDataPacket(&AD855XDataTemp, FUNCTION_CHANGE_SENSE_CURRENT, NULL, value);

	for(i=0; i<38; i++)
	{
		AD855XDin = 0;
		ADuC841Delay(30);
		AD855XDin = 1;
		if(AD855XDataTemp.SerialBit[i]==1)
		{
			ADuC841Delay(80);
		}
		else
		{
			ADuC841Delay(3);
		}
		
	}
	AD855XDin = 0;

	EA = 1;	 /* enable interrupts */
}
void AD855XSimulateParameterValue(unsigned char parameter, unsigned char value)
{
	xdata AD855XDataStructure AD855XDataTemp;

	unsigned char i;

	EA = 0;	 /* clear interrupts */

	AD855XDataPacket(&AD855XDataTemp, FUNCTION_SIMULATE_PARAMETER_VALUE, parameter, value);

	for(i=0; i<38; i++)
	{
		AD855XDin = 0;
		ADuC841Delay(30);
		AD855XDin = 1;
		if(AD855XDataTemp.SerialBit[i]==1)
		{
			ADuC841Delay(80);
		}
		else
		{
			ADuC841Delay(3);
		}
		
	}
	AD855XDin = 0;

	EA = 1;	 /* enable interrupts */
}
void AD855XProgramParameterValue(unsigned char parameter, unsigned char value)
{
	xdata AD855XDataStructure AD855XDataTemp;

	unsigned char i;
	unsigned char j;
	unsigned char tempValue=0x01;

	EA = 0;	 /* clear interrupts */

	for(j=0; j<8; j++)
	{
		if((value&tempValue) == tempValue)
		{
			AD855XDataPacket(&AD855XDataTemp, FUNCTION_PROGRAM_PARAMETER_VALUE, parameter, tempValue);
			/*for test only
			for(i=0; i<38; i++)
			{
				ADuC841SendChar(0x30+(AD855XDataTemp.SerialBit[i]));
			}
			ADuC841SendChar(0x0D);
			ADuC841SendChar(0x0A);
			*/
			for(i=0; i<38; i++)
			{
				AD855XDin = 0;
				ADuC841Delay(30);
				AD855XDin = 1;
				if(AD855XDataTemp.SerialBit[i]==1)
				{
					ADuC841Delay(80);
				}
				else
				{
					ADuC841Delay(3);
				}
			}
			AD855XDin = 0;	
		}
		tempValue=tempValue<<1;

		ADuC841Delay(8000);  // delay at least 1ms
	}
	EA = 1;	 /* enable interrupts */
}

void AD855XBLOWMASTERFUSE(void)
{
	xdata AD855XDataStructure AD855XDataTemp;

	unsigned char i;

	EA = 0;	 /* clear interrupts */

	AD855XDataPacket(&AD855XDataTemp, FUNCTION_PROGRAM_PARAMETER_VALUE, PARAMETER_OTHER_FUNCTIONS, MASTER_FUSE);

	for(i=0; i<38; i++)
	{
		AD855XDin = 0;
		ADuC841Delay(30);
		AD855XDin = 1;
		if(AD855XDataTemp.SerialBit[i]==1)
		{
			ADuC841Delay(80);
		}
		else
		{
			ADuC841Delay(3);
		}
		
	}
	AD855XDin = 0;

	ADuC841Delay(8000);  // delay at least 1ms

	EA = 1;	 /* enable interrupts */
}
// only for AD8556
void AD855XBlowParityFuse(void)
{
	xdata AD855XDataStructure AD855XDataTemp;

	unsigned char i;

	EA = 0;	 /* clear interrupts */

	AD855XDataPacket(&AD855XDataTemp, FUNCTION_PROGRAM_PARAMETER_VALUE, PARAMETER_OTHER_FUNCTIONS, PARITY_FUSE);

	for(i=0; i<38; i++)
	{
		AD855XDin = 0;
		ADuC841Delay(30);
		AD855XDin = 1;
		if(AD855XDataTemp.SerialBit[i]==1)
		{
			ADuC841Delay(80);
		}
		else
		{
			ADuC841Delay(3);
		}
		
	}
	AD855XDin = 0;

	ADuC841Delay(8000);  // delay at least 1ms

	EA = 1;	 /* enable interrupts */
}

unsigned char AD855XReadParameterValue(unsigned char parameter)
{
	xdata AD855XDataStructure AD855XDataTemp;

	unsigned char i;
	unsigned char tempValue = 0x00;

	EA = 0;	 /* clear interrupts */

	AD855XDataPacket(&AD855XDataTemp, FUNCTION_READ_PARAMETER_VALUE, parameter, NULL);

	for(i=0; i<38; i++)
	{
		AD855XDin = 0;
		ADuC841Delay(30);
		AD855XDin = 1;
		if(AD855XDataTemp.SerialBit[i]==1)
		{
			ADuC841Delay(80);
		}
		else
		{
			ADuC841Delay(3);
		}
		
	}
	
	for(i=0; i<8; i++)
	{
		AD855XDin = 0;
		ADuC841Delay(30);
		tempValue = (tempValue<<1) | AD855XDOUT;				//read
		AD855XDin = 1;
		ADuC841Delay(30);
	
	}

	AD855XDin = 0;

	
	EA = 1;	 /* enable interrupts */
	return tempValue;
}
