/************************************************************************//**
*   @file:    AD855X.h  
*   @brief:   Basic operations for AD855X
*   
*****************************************************************************
*   Copyright (c) 2011 Analog Devices Inc.
*   License Agreement (if necessary)
*   Legal Disclaimer (if necessary)
*
****************************************************************************/

#ifndef __C_HEADER_AD855X_H__
#define __C_HEADER_AD855X_H__

/***************************************************************************/
/************************ AD855X Data Field Definitions ********************/
/***************************************************************************/
#define START								0x801

#define FUNCTION_CHANGE_SENSE_CURRENT		0x00
#define FUNCTION_SIMULATE_PARAMETER_VALUE	0x01   
#define FUNCTION_PROGRAM_PARAMETER_VALUE	0x02
#define FUNCTION_READ_PARAMETER_VALUE		0x03

#define PARAMETER_SECOND_STAGE_GAIN_CODE	0x00
#define LOW_CURRENT							0x01
#define HIGH_CURRENT						0x00

#define PARAMETER_FIRST_STAGE_GAIN_CODE		0x01

#define PARAMETER_OUTPUT_OFFSET_CODE		0x02

#define PARAMETER_OTHER_FUNCTIONS			0x03
#define MASTER_FUSE							0x01	
#define PARITY_FUSE							0x04	// only for AD8556				

#define DUMMY								0x02

#define STOP								0x7FE

#define NULL								0x00

/***************************************************************************/
/************************** Types Declarations *****************************/
/***************************************************************************/

typedef struct AD855XDATASTRUCTURE
{
	unsigned start:12;
	unsigned function:2;
	unsigned parameter:2;
	unsigned dummy:2;
	unsigned value:8;
	unsigned end:12;
	unsigned char SerialBit[38];
}AD855XDataStructure;

sbit AD855XDin = 0xB3;	/* P3.3 is used as AD8577 DIN pin */
sbit AD855XDOUT = 0xB2;	/* P3.2 is used as AD8577 DOUT pin */

/***************************************************************************/
/************************ Functions Declarations ***************************/
/***************************************************************************/
void AD855XDataPacket(AD855XDataStructure* AD855XData, unsigned char function, unsigned char parameter, unsigned char value);
void AD855XChangeSenseCurrent(unsigned char value);
void AD855XSimulateParameterValue(unsigned char parameter, unsigned char value);
void AD855XProgramParameterValue(unsigned char parameter, unsigned char value);
void AD855XBlowParityFuse(void);
void AD855XBlowMasterFuse(void);
unsigned char AD855XReadParameterValue(unsigned char parameter);



#endif /*__C_HEADER_AD855X_H__*/